/* dumpmem - a small utility to execute a binary and dump its memory */ 
/* contents before it begins executing.                              */ 
/*                                                                   */ 
/* Muli Ben-Yehuda <mulix@mulix.org>                                 */ 
/* Licensed under the GPL V2                                         */ 
/*                                                                   */ 
/* $Id: maps.c,v 1.1 2002/11/13 11:34:13 mulix Exp $              */ 

#include <stdio.h> 
#include <string.h> 
#include <stdlib.h> 
#include <unistd.h> 
#include <errno.h> 
#include <fcntl.h>
#include <sys/stat.h>
#include <sys/types.h>

#include "util.h" 
#include "maps.h" 

/* map constructor */ 
static struct map* make_map(unsigned long begin, unsigned long end)
{
	struct map* m = malloc(sizeof(*m)); 

	if (!m)
		fatal("malloc"); 

	m->begin = begin; m->end = end; 
	m->next = NULL; 
	return m; 
}

/* get the maps for the process */ 
void get_maps(pid_t proc, struct map** m)
{
	/* open the maps file for this pid */ 
	char name[32]; 
	char buf[32000]; 
	int fd; 
	int ret; 
	int available = sizeof(buf); 
	struct map* head = NULL, *curr; 
	char* b, *e; 
	unsigned long begin, end; 

	(void)snprintf(name, sizeof(name), "/proc/%d/maps", (int)proc); 

	if (!(fd = open(name, O_RDONLY)))
		fatal("open %s: %s", name, strerror(errno)); 

	/* we make a single read and hope to read everything. Shaky at best */ 
	if ((ret = read(fd, buf, available - 1)) < 0)
		fatal("read %s: %s", name, strerror(errno)); 

	buf[ret] = '\0'; 

	close(fd); 
	/* we've read 'ret' bytes, parse it. file format: */ 
/* 
   08048000-08053000 r-xp 00000000 03:06 352128     /bin/ls
   08053000-08054000 rw-p 0000b000 03:06 352128     /bin/ls
   00000000-40013000 r-xp 00000000 03:06 512051     /lib/ld-2.2.5.so
   40013000-40014000 rw-p 00013000 03:06 512051     /lib/ld-2.2.5.so
   bffff000-c0000000 rwxp 00000000 00:00 0
*/ 

	/* this parser is ugly. rewrite */ 
	b = buf; 
	do { 
		/* find the first '-' */ 
		if (!(e = strchr(b, '-')))
			fatal("invalid maps file format!"); 
		
		*e = '\0'; 
		begin = strtoul(b, NULL, 16); 
		if (!*(b = e + 1))
			fatal("invalid maps file format!"); 
		
		if (!(e = strchr(b, ' ')))
			fatal("invalid maps file format!"); 
		
		*e = '\0'; 
		end = strtoul(b, NULL, 16); 

		/* make the map */ 
		if (!head)
			curr = head = make_map(begin, end); 
		else {
			curr->next = make_map(begin, end); 
			curr = curr->next; 
		}
		
		/* look for a \n and another line */ 
		if (!(b = strchr(e + 1, '\n')))
			break; 
		/* if we found a NULL */ 
		if (!*++b)
			break; 
	} while (1); 
	
	*m = head; 
}

