#!/usr/bin/env python2

# this is a simple test script for dumpmem
#
# Muli Ben-Yehuda <mulix@mulix.org>
# Licensed under the GPL V2
#
# $Id: tester.py,v 1.4 2002/12/14 09:13:15 mulix Exp $

import os
import string

class dumpmem_test:
    "a base test class for dumpmem tests"

    cmdline_flags = [] 
    
    def run(self):
        raise "%s does not implement run()" % (self.name())

    def __init__(self, output_format = "default"):
        if (output_format == "default"):
            pass
        elif (output_format == "raw"):
            self.cmdline_flags.append("-r")
        else:
            e = "%s has bad output_format: %s" % (self.name(), output_format)
            raise e

    class SpawnError(Exception):   # the exception we throw if any error occurs
        def __init__(self, value): # constructor
            self.value = value

        def __str__(self):         # string representation
            return `self.value`

    def mkcmd(self, str):
        """ utility function, make a list from a string """
        return string.split(str)

    def name(self):
        return self.__class__.__name__

    def run_command(self, com):
        """ run a command in the local directory """
        print "%s: %s" % (self.name(), string.join(com))
        rc = os.spawnvp(os.P_WAIT, com[0], com) 
        if (rc != 0):
            print com
            error = string.join(com) + " failed"
            raise self.SpawnError, error

    def run_dumpmem(self, args):
        """run dumpmem [cmdline_flags] args"""
        com = ["./dumpmem"]
        com.extend(self.cmdline_flags)
        com.extend(args)
        self.run_command(com)

    pass


class check_consistent_dump(dumpmem_test):
    "check that two dumps of the same executable are consistent"

    def run(self):
        self.run_dumpmem(self.mkcmd("-o dump1 ls")) 
        self.run_dumpmem(self.mkcmd("-o dump2 ls"))
        self.run_command(self.mkcmd("cmp dump1 dump2"))
        self.run_command(self.mkcmd("rm dump1 dump2"))

if __name__ == "__main__":
    tests = [check_consistent_dump("default"),
             check_consistent_dump("raw")]

    for test in tests:
        test.run()
